@@ -65,7 +65,11 @@ public:
     void endConfig();
 
     int getRunOnStartUp() const     {   return m_iRunOnStartUp;  }
-    void setRunOnStartUp(int r)   {   m_iRunOnStartUp = r;     }
+    void setRunOnStartUp(int r)   
+    {
+        setDetached(r == 3);
+        m_iRunOnStartUp = r;     
+    }
 
     void setRLimits(const RLimits *pRLimits);
     const RLimits *getRLimits() const    {   return &m_rlimits;   }
@@ -81,7 +85,7 @@ public:
     int isPhpHandler() { return m_iPhpHandler ; }
     void setPhpHandler(int v)       { m_iPhpHandler = v;    }
     
-    int checkExtAppSelfManagedAndFixEnv();
+    int checkExtAppSelfManagedAndFixEnv(int maxIdleTime);
     int config(const XmlNode *pNode);
     void configExtAppUserGroup(const XmlNode *pNode, int iType, char *sHomeDir);
 };
