@@ -144,7 +144,7 @@ void ClientInfo::setAddr(const struct sockaddr *pAddr)
     else
     {
         len = 24;
-        strLen = 43;
+        strLen = 41;
     }
 
 #if 0
@@ -175,22 +175,9 @@ void ClientInfo::setAddr(const struct sockaddr *pAddr)
     m_sAddr.prealloc(strLen);
     if (m_sAddr.buf())
     {
-        char *p = m_sAddr.buf();
-        if (AF_INET6 == pAddr->sa_family)
-        {
-            *p++ = '[';
-            strLen -= 2;
-        }
-        inet_ntop(pAddr->sa_family, ((char * )pAddr) + ((len >> 1) - 4),
-                  p, strLen);
-        int l = strlen(p);
-        if (AF_INET6 == pAddr->sa_family)
-        {
-            *(p + l) = ']';
-            *(p + l + 1) = '\0';
-            l += 2;
-        }
-        m_sAddr.setLen(l);
+        inet_ntop(pAddr->sa_family, ((char *)pAddr) + ((len >> 1) - 4),
+                  m_sAddr.buf(), strLen);
+        m_sAddr.setLen(strlen(m_sAddr.c_str()));
     }
     memset(&m_iConns, 0, (char *)(&m_lastConnect + 1) - (char *)&m_iConns);
     m_iAccess = 1;
