@@ -38,6 +38,7 @@
 #include <main/mainserverconfig.h>
 #include <main/plainconf.h>
 #include <main/serverinfo.h>
+#include <quic/quicengine.h>
 #include <shm/lsshmpool.h>
 #include <socket/coresocket.h>
 #include <util/datetime.h>
@@ -80,7 +81,7 @@
 /***
  * Do not change the below format, it will be set correctly while packing the code
  */
-#define BUILDTIME  " (built: Wed Aug 21 20:09:04 UTC 2019)"
+#define BUILDTIME  " (built: Tue Sep 10 15:56:44 UTC 2019)"
 
 #define GlobalServerSessionHooks (LsiApiHooks::getServerSessionHooks())
 
@@ -958,6 +959,8 @@ int LshttpdMain::init(int argc, char *argv[])
             startAdminSocket();
     }
 
+    QuicEngine::setpid(getpid());
+    
 #ifndef IS_LSCPD
     ret = config();
     if (ret && !m_iConfTestMode)
@@ -1040,6 +1043,7 @@ int LshttpdMain::init(int argc, char *argv[])
         GlobalServerSessionHooks->runCallbackNoParam(LSI_HKPT_WORKER_INIT,
                 NULL);
 
+    
 
     return 0;
 }
@@ -1247,6 +1251,7 @@ void LshttpdMain::onNewChildStart(ChildProc * pProc)
 #endif
 
     LsShmPool::setPid(pProc->m_pid);
+    QuicEngine::setpid(pProc->m_pid);
 }
 
 int LshttpdMain::startChild(ChildProc *pProc)
@@ -1782,8 +1787,11 @@ static long getProcessStartTime(int pid)
     if (sysctl(mib, 4, &kp, &len, NULL, 0) != 0) {
         return -1;
     }
-
+#if defined(__FreeBSD__ ) || defined(__NetBSD__) || defined(__OpenBSD__) 
     return kp.ki_start.tv_sec;
+#else
+    return kp.kp_proc.p_un.__p_starttime.tv_sec;
+#endif
 }
 #endif
 
