@@ -17,12 +17,14 @@
 *****************************************************************************/
 #include "ssiengine.h"
 
+#include <ls.h>
 #include "ssiscript.h"
 #include "ssiruntime.h"
 #include "ssiconfig.h"
 
 #include <http/handlertype.h>
 #include <http/httpcgitool.h>
+#include <http/httpmethod.h>
 #include <http/httpsession.h>
 #include <http/httpstatuscode.h>
 #include <http/requestvars.h>
@@ -31,57 +33,64 @@
 #include <util/httputil.h>
 #include <util/ienv.h>
 
+#include <ctype.h>
 #include <stdio.h>
 
-SSIEngine::SSIEngine()
+SsiEngine::SsiEngine()
     : HttpHandler(HandlerType::HT_SSI)
 {
 }
 
 
-SSIEngine::~SSIEngine()
+SsiEngine::~SsiEngine()
 {
 }
 
 
-const char *SSIEngine::getName() const
+const char *SsiEngine::getName() const
 {
     return "ssi";
 }
 
 
-void SSIEngine::printError(HttpSession *pSession, char *pError)
+void SsiEngine::printError(HttpSession *pSession, const char *pError)
 {
-    char achBuf[] = "[an error occurred while processing this directive]\n";
+    SsiRuntime *pRuntime = pSession->getSsiRuntime();
+    int len;
     if (!pError)
-        pError = achBuf;
+    {
+        pError = pRuntime->getConfig()->getErrMsg()->c_str();
+        len = pRuntime->getConfig()->getErrMsg()->len();
+    }
+    else
+        len = strlen(pError);
 
-    pSession->appendDynBody(pError, strlen(pError));
+    pSession->appendDynBody(pError, len);
 }
 
 
-int SSIEngine::startExecute(HttpSession *pSession,
-                            SSIScript *pScript)
+int SsiEngine::beginExecute(HttpSession *pSession,
+                            const SsiScript *pScript)
 {
     if (!pScript)
         return SC_500;
-    SSIRuntime *pRuntime = pSession->getReq()->getSSIRuntime();
+    SsiRuntime *pRuntime = pSession->getSsiRuntime();
     if (!pRuntime)
     {
-        char ct[] = "text/html";
-        HttpReq *pReq = pSession->getReq();
-        pRuntime = new SSIRuntime();
-        if (!pRuntime)
+        if (pSession->setupSsiRuntime() == -1)
             return SC_500;
-        pRuntime->init();
-        pRuntime->initConfig(pReq->getSSIConfig());
-        pReq->setSSIRuntime(pRuntime);
+
+        SsiRuntime *pRuntime = pSession->getSsiRuntime();
+        HttpReq *pReq = pSession->getReq();
+        char ct[] = "text/html";
         pSession->getResp()->reset();
         //pSession->getResp()->prepareHeaders( pReq );
         //pSession->setupChunkOS( 0 );
-        HttpCgiTool::processContentType(pSession, ct, 9);
-//        pSession->setupRespCache();
-        if (pReq->isXbitHackFull())
+        if (!pSession->getResp()->getRespHeaders()
+            .isHeaderSet(HttpRespHeaders::H_CONTENT_TYPE))
+            HttpCgiTool::processContentType( pSession, ct , 9);
+        //pSession->setupRespCache();
+        if (pReq->isXbitHackFull() || pRuntime->getConfig()->isLastModOn())
             pSession->getResp()->appendLastMod(pReq->getLastMod());
         int status = pReq->getStatusCode();
         if ((status >= SC_300) && (status < SC_400))
@@ -89,20 +98,62 @@ int SSIEngine::startExecute(HttpSession *pSession,
             if (pReq->getLocation() != NULL)
                 pSession->addLocationHeader();
         }
-        pSession->setupGzipFilter();
+        if (!pSession->isNoRespBody())
+        {
+            if (pSession->setupRespBodyBuf() != -1)
+                pSession->setupGzipFilter();
+        }
         pReq->andGzip(~GZIP_ENABLED);    //disable GZIP
     }
-    if (pRuntime->push(pScript) == -1)
-        return SC_500;
-    pSession->getReq()->backupPathInfo();
-    pScript->resetRuntime();
-    return resumeExecute(pSession);
+    else
+        pSession->setupRespBodyBuf();
+    pSession->prepareSsiStack(pScript);
+
+    return execute(pSession);
+
+
+
+
+
+
+//     if (!pRuntime)
+//     {
+//         char ct[] = "text/html";
+//         HttpReq *pReq = pSession->getReq();
+//         pRuntime = new SsiRuntime();
+//         if (!pRuntime)
+//             return SC_500;
+//         pRuntime->init();
+//         pRuntime->initConfig(pReq->getSsiConfig());
+//         pReq->setSsiRuntime(pRuntime);
+//         pSession->getResp()->reset();
+//         //pSession->getResp()->prepareHeaders( pReq );
+//         //pSession->setupChunkOS( 0 );
+//         HttpCgiTool::processContentType(pReq, pSession->getResp(),
+//                                         ct , 9);
+// //        pSession->setupRespCache();
+//         if (pReq->isXbitHackFull())
+//             pSession->getResp()->appendLastMod(pReq->getLastMod());
+//         int status = pReq->getStatusCode();
+//         if ((status >= SC_300) && (status < SC_400))
+//         {
+//             if (pReq->getLocation() != NULL)
+//                 pSession->addLocationHeader();
+//         }
+//         pSession->setupGzipFilter();
+//         pReq->andGzip(~GZIP_ENABLED);    //disable GZIP
+//     }
+//     if (pRuntime->push(pScript) == -1)
+//         return SC_500;
+//     pSession->getReq()->backupPathInfo();
+//     pScript->resetRuntime();
+//     return resumeExecute(pSession);
 }
 
 
-int SSIEngine::updateSSIConfig(HttpSession *pSession,
-                               SSIComponent *pComponent,
-                               SSIRuntime *pRuntime)
+int SsiEngine::updateSsiConfig(HttpSession *pSession,
+                               const SsiComponent *pComponent,
+                               SsiRuntime *pRuntime)
 {
     SubstItem *pItem = (SubstItem *)pComponent->getFirstAttr();
     char achBuf[4096];
@@ -116,18 +167,18 @@ int SSIEngine::updateSSIConfig(HttpSession *pSession,
         len = 4096;
         switch (attr)
         {
-        case SSI_ECHOMSG:
-        case SSI_ERRMSG:
-        case SSI_TIMEFMT:
+        case SSI_ATTR_ECHOMSG:
+        case SSI_ATTR_ERRMSG:
+        case SSI_ATTR_TIMEFMT:
             RequestVars::appendSubst(pItem, pSession, p, len, 0, NULL);
-            if (attr == SSI_ERRMSG)
+            if (attr == SSI_ATTR_ERRMSG)
                 pRuntime->getConfig()->setErrMsg(achBuf, p - achBuf);
-            else if (attr == SSI_ECHOMSG)
+            else if (attr == SSI_ATTR_ECHOMSG)
                 pRuntime->getConfig()->setEchoMsg(achBuf, p - achBuf);
             else
                 pRuntime->getConfig()->setTimeFmt(achBuf, p - achBuf);
             break;
-        case SSI_SIZEFMT:
+        case SSI_ATTR_SIZEFMT:
             if (pItem->getType() == REF_STRING)
             {
                 const AutoStr2 *pStr = pItem->getStr();
@@ -144,7 +195,8 @@ int SSIEngine::updateSSIConfig(HttpSession *pSession,
 }
 
 
-int SSIEngine::processEcho(HttpSession *pSession, SSIComponent *pComponent)
+int SsiEngine::processEcho(HttpSession *pSession,
+                           const SsiComponent *pComponent)
 {
     SubstItem *pItem = (SubstItem *)pComponent->getFirstAttr();
     char achBuf1[8192];
@@ -152,7 +204,7 @@ int SSIEngine::processEcho(HttpSession *pSession, SSIComponent *pComponent)
     char *p;
     int len;
     int attr;
-    int encode = SSI_ENC_ENTITY;
+    int encode = SSI_ATTR_ENC_ENTITY;
     while (pItem)
     {
         attr = pItem->getSubType();
@@ -160,21 +212,21 @@ int SSIEngine::processEcho(HttpSession *pSession, SSIComponent *pComponent)
         len = 8192;
         switch (attr)
         {
-        case SSI_ECHO_VAR:
+        case SSI_ATTR_ECHO_VAR:
             if ((pItem->getType() == REF_DATE_LOCAL) ||
                 (pItem->getType() == REF_LAST_MODIFIED) ||
                 (pItem->getType() == REF_DATE_GMT))
-                memccpy(p, pSession->getReq()->getSSIRuntime()
+                memccpy(p, pSession->getSsiRuntime()
                         ->getConfig()->getTimeFmt()->c_str(), 0, 4096);
             RequestVars::appendSubst(pItem, pSession, p, len,
-                                     0, pSession->getReq()->getSSIRuntime()->getRegexResult());
-            if (encode == SSI_ENC_URL)
+                                     0, pSession->getSsiRuntime()->getRegexResult());
+            if (encode == SSI_ATTR_ENC_URL)
             {
                 len = HttpUtil::escape(achBuf1, p - achBuf1,
                                        achBuf, 40960);
                 p = achBuf;
             }
-            else if (encode == SSI_ENC_ENTITY)
+            else if (encode == SSI_ATTR_ENC_ENTITY)
             {
                 len = HttpUtil::escapeHtml(achBuf1, p, achBuf, 40960);
                 p = achBuf;
@@ -187,9 +239,9 @@ int SSIEngine::processEcho(HttpSession *pSession, SSIComponent *pComponent)
             if (len > 0)
                 pSession->appendDynBody(p, len);
             break;
-        case SSI_ENC_NONE:
-        case SSI_ENC_URL:
-        case SSI_ENC_ENTITY:
+        case SSI_ATTR_ENC_NONE:
+        case SSI_ATTR_ENC_URL:
+        case SSI_ATTR_ENC_ENTITY:
             encode = attr;
             break;
         }
@@ -200,7 +252,8 @@ int SSIEngine::processEcho(HttpSession *pSession, SSIComponent *pComponent)
 }
 
 
-int SSIEngine::processExec(HttpSession *pSession, SSIComponent *pComponent)
+int SsiEngine::processExec(HttpSession *pSession,
+                           const SsiComponent *pComponent)
 {
     if (pSession->getReq()->isIncludesNoExec())
     {
@@ -213,8 +266,8 @@ int SSIEngine::processExec(HttpSession *pSession, SSIComponent *pComponent)
 }
 
 
-int SSIEngine::processFileAttr(HttpSession *pSession,
-                               SSIComponent *pComponent)
+int SsiEngine::processFileAttr(HttpSession *pSession,
+                               const SsiComponent *pComponent)
 {
     SubstItem *pItem = (SubstItem *)pComponent->getFirstAttr();
     if (!pItem)
@@ -224,23 +277,25 @@ int SSIEngine::processFileAttr(HttpSession *pSession,
     char achBuf[4096];
     char *p = achBuf;
     len = 4096;
-    if ((attr != SSI_INC_FILE) && (attr != SSI_INC_VIRTUAL))
+    if ((attr != SSI_ATTR_INC_FILE) && (attr != SSI_ATTR_INC_VIRTUAL))
         return 0;
-    SSIRuntime *pRuntime = pSession->getReq()->getSSIRuntime();
+    SsiRuntime *pRuntime = pSession->getSsiRuntime();
     RequestVars::appendSubst(pItem, pSession, p, len,
                              0, pRuntime->getRegexResult());
     {
         if (achBuf[0] == 0)
         {
-            len = snprintf(achBuf, 4096,
-                           "[an error occurred while processing this directive]\n");
-            pSession->appendDynBody(achBuf, len);
+            pSession->appendDynBody(pRuntime->getConfig()->getErrMsg()->c_str(),
+                                    pRuntime->getConfig()->getErrMsg()->len());
+//             len = snprintf(achBuf, 4096,
+//                            "[an error occurred while processing this directive]\n");
+//             pSession->appendDynBody(achBuf, len);
             return 0;
         }
         HttpReq *pReq = pSession->getReq();
         const char *pURI ;
         const char *p1;
-        if ((attr == SSI_INC_FILE) || (achBuf[0] != '/'))
+        if ((attr == SSI_ATTR_INC_FILE) || (achBuf[0] != '/'))
         {
             pURI = pReq->getRealPath()->c_str();
             p1 =  pURI + pReq->getRealPath()->len();
@@ -265,7 +320,7 @@ int SSIEngine::processFileAttr(HttpSession *pSession,
         pSession->appendDynBody("[error: stat() failed!\n", 23);
         return 0;
     }
-    if (pComponent->getType() == SSIComponent::SSI_FSize)
+    if (pComponent->getType() == SsiComponent::SSI_FSize)
     {
         long long size = st.st_size;
         len = snprintf(achBuf, 1024, "%lld", size);
@@ -282,21 +337,69 @@ int SSIEngine::processFileAttr(HttpSession *pSession,
 }
 
 
-int SSIEngine::processSubReq(HttpSession *pSession, SubstItem *pItem)
+int SsiEngine::toLocalAbsUrl(HttpSession *pSession, const char *pOrgUrl,
+                             int urlLen, char *pAbsUrl, int absLen)
+{
+    HttpReq *pReq = pSession->getReq();
+    const char *pURI = pReq->getURI();
+    const char *p1 = pOrgUrl;
+    if ((strncasecmp(p1, "http", 4) == 0) &&
+        ((*(p1 + 4) == ':') ||
+         (((*(p1 + 4) | 0x20) == 's') && (*(p1 + 5) == ':'))))
+    {
+        p1 += 5;
+        if (*p1 == ':')
+            ++p1;
+        p1 += 2;
+        if (strncasecmp(p1, pSession->getReq()->getHeader(HttpHeader::H_HOST),
+                        pSession->getReq()->getHeaderLen(HttpHeader::H_HOST)) == 0)
+        {
+            p1 += pSession->getReq()->getHeaderLen(HttpHeader::H_HOST);
+            if (*p1 == ':')
+            {
+                const char *p = p1 + 1;
+                while (isdigit(*p))
+                    ++p;
+                if (*p == '/')
+                    p1 = p;
+            }
+        }
+        else
+            return urlLen;
+        memmove(pAbsUrl, p1, pOrgUrl + urlLen - p1 + 1);
+        return pOrgUrl + urlLen - p1;
+    }
+    p1 = pURI + pReq->getURILen() - pReq->getPathInfoLen();
+    while ((p1 > pURI) && p1[-1] != '/')
+        --p1;
+    int prefix_len = p1 - pURI;
+    if (absLen <= urlLen + prefix_len)
+        return -1;
+    memmove(pAbsUrl + prefix_len, pOrgUrl, urlLen);
+    memmove(pAbsUrl, pURI, prefix_len);
+    pAbsUrl[urlLen + prefix_len] = 0;
+    return urlLen + prefix_len;
+}
+
+
+int SsiEngine::processSubReq(HttpSession *pSession, SubstItem *pItem)
 {
     char achBuf[40960];
     char *p;
     int len;
     int attr;
+    const char *pQs = NULL;
+    int iQsLen = 0;
     if (!pItem)
         return 0;
-    SSIRuntime *pRuntime = pSession->getReq()->getSSIRuntime();
+    SsiRuntime *pRuntime = pSession->getSsiRuntime();
+    SsiStack   *pStack = pSession->getSsiStack();
     attr = pItem->getSubType();
     p = achBuf;
     len = 40960;
     switch (attr)
     {
-    case SSI_INC_FILE:
+    case SSI_ATTR_INC_FILE:
         {
             HttpReq *pReq = pSession->getReq();
             memmove(p, pReq->getURI(), pReq->getURILen());
@@ -307,13 +410,15 @@ int SSIEngine::processSubReq(HttpSession *pSession, SubstItem *pItem)
             len -= p - achBuf;
             break;
         }
-    case SSI_EXEC_CGI:
-        pRuntime->requireCGI();
+    case SSI_ATTR_EXEC_CGI:
+        pQs = pSession->getReq()->getQueryString();
+        iQsLen = pSession->getReq()->getQueryStringLen();
+        pStack->requireCGI();
         break;
-    case SSI_EXEC_CMD:
-        pRuntime->requireCmd();
+    case SSI_ATTR_EXEC_CMD:
+        pStack->requireCmd();
         p += snprintf(achBuf, 40960, "%s",
-                      pRuntime->getCurrentScript()->getPath());
+                      pStack->getScript()->getPath()->c_str());
         while (p[-1] != '/')
             --p;
         *p++ = '&';
@@ -328,13 +433,14 @@ int SSIEngine::processSubReq(HttpSession *pSession, SubstItem *pItem)
     }
     RequestVars::appendSubst(pItem, pSession, p, len,
                              0, pRuntime->getRegexResult());
-    if (attr == SSI_INC_FILE)
+    *p = 0;
+    if (attr == SSI_ATTR_INC_FILE)
     {
         if (strstr(achBuf, "/../") != NULL)
 
             return 0;
     }
-    else if (attr == SSI_EXEC_CMD)
+    else if (attr == SSI_ATTR_EXEC_CMD)
     {
 
         if (pSession->execExtCmd(achBuf, p - achBuf) == 0)
@@ -347,54 +453,97 @@ int SSIEngine::processSubReq(HttpSession *pSession, SubstItem *pItem)
 
         //return 0;
     }
-    if ((achBuf[0] != '/') && (attr != SSI_EXEC_CMD))
+    if ((achBuf[0] != '/')) // && (attr != SSI_ATTR_EXEC_CMD))
     {
         if (achBuf[0] == 0)
+            len = -1;
+        else
         {
-            len = snprintf(achBuf, 40960,
-                           "[an error occurred while processing this directive]\n");
-            pSession->appendDynBody(achBuf, len);
+            len = toLocalAbsUrl(pSession, achBuf, p - achBuf, achBuf, sizeof(achBuf));
+            p = &achBuf[len];
+            *p = 0;
+        }
+        if (len == -1)
+        {
+            pSession->appendDynBody(pRuntime->getConfig()->getErrMsg()->c_str(),
+                                    pRuntime->getConfig()->getErrMsg()->len());
             return 0;
         }
-        HttpReq *pReq = pSession->getReq();
-        const char *pURI = pReq->getURI();
-        const char *p1 = pURI + pReq->getURILen() - pReq->getPathInfoLen();
-        while ((p1 > pURI) && p1[-1] != '/')
-            --p1;
-        int prefix_len = p1 - pURI;
-        memmove(&achBuf[prefix_len], achBuf, p - achBuf);
-        memmove(achBuf, pReq->getURI(), prefix_len);
-        p += prefix_len;
     }
     if (achBuf[0] == '/')
+        return startSubSession(pSession, achBuf, p - achBuf, pQs, iQsLen);
+    else
     {
-        pSession->getReq()->setLocation(achBuf, p - achBuf);
-        pSession->changeHandler();
-        pSession->continueWrite();
-        return -2;
+        LS_INFO(pSession->getLogSession(),
+                "Cannot include None-local URL or configured proxy backend: %s",
+                achBuf);
+        printError(pSession, NULL);
     }
     return 0;
 }
 
 
-int SSIEngine::processInclude(HttpSession *pSession,
-                              SSIComponent *pComponent)
+int SsiEngine::startSubSession(HttpSession *pSession,
+                               const char *pUri, int uriLen,
+                               const char *pQs, int qsLen)
+{
+    lsi_subreq_t subSessionInfo;
+    memset(&subSessionInfo, 0, sizeof(lsi_subreq_t));
+    subSessionInfo.m_pUri   = pUri;
+    subSessionInfo.m_uriLen = uriLen;
+    subSessionInfo.m_pQs    = pQs;
+    subSessionInfo.m_qsLen  = qsLen;
+    subSessionInfo.m_method = HttpMethod::HTTP_GET;
+    subSessionInfo.m_flag |= SUB_REQ_SETREFERER;
+
+    return startSubSession(pSession, &subSessionInfo);
+}
+
+
+int SsiEngine::startSubSession(HttpSession *pSession,
+                               lsi_subreq_t *pSubSessionInfo)
+{
+    HttpSession *pSubSession = pSession->newSubSession(pSubSessionInfo);
+    if (!pSubSession)
+    {
+        printError(pSession, NULL);
+        return -1;
+    }
+
+    pSubSession->getStream()->setFlag(HIO_FLAG_PASS_SETCOOKIE, 1);
+    pSubSession->setSsiRuntime(pSession->getSsiRuntime());
+    pSubSession->setFlag(HSF_NO_ERROR_PAGE);
+    int ret = pSubSession->execSubSession();
+    return processSubSessionRet(ret, pSession, pSubSession);
+}
+
+
+int SsiEngine::processInclude(HttpSession *pSession,
+                              const SsiComponent *pComponent)
 {
     SubstItem *pItem = (SubstItem *)pComponent->getFirstAttr();
+    if (pSession->getSsiStack()->getDepth() > 9)
+    {
+        LS_DBG_H(pSession->getLogSession(),
+                 "maximum include depth 10 reached, skip."
+                  );
+        printError(pSession, NULL);
+        return 0;
+    }
     return processSubReq(pSession, pItem);
 }
 
 
-class SSIEnv : public IEnv
+class SsiEnv : public IEnv
 {
     HttpSession *m_pSession;
 
 
-    SSIEnv(const SSIEnv &rhs);
-    void operator=(const SSIEnv &rhs);
+    SsiEnv(const SsiEnv &rhs);
+    void operator=(const SsiEnv &rhs);
 public:
-    SSIEnv(HttpSession *pSession) : m_pSession(pSession)    {};
-    ~SSIEnv()   {};
+    SsiEnv(HttpSession *pSession) : m_pSession(pSession)    {};
+    ~SsiEnv()   {};
     int add(const char *name, const char *value)
     {   return IEnv::add(name, value);    }
 
@@ -406,7 +555,7 @@ public:
 };
 
 
-int SSIEnv::add(const char *name, size_t nameLen,
+int SsiEnv::add(const char *name, size_t nameLen,
                 const char *value, size_t valLen)
 {
     char achBuf[40960];
@@ -424,7 +573,7 @@ int SSIEnv::add(const char *name, size_t nameLen,
 }
 
 
-int SSIEnv::add(const char *buf, size_t len)
+int SsiEnv::add(const char *buf, size_t len)
 {
     char achBuf[40960];
     int ret = HttpUtil::escapeHtml(buf, buf + len, achBuf, 40960);
@@ -434,7 +583,7 @@ int SSIEnv::add(const char *buf, size_t len)
 }
 
 
-int SSIEnv::addVar(int var_id)
+int SsiEnv::addVar(int var_id)
 {
     char achBuf[4096];
     const char *pName;
@@ -443,16 +592,16 @@ int SSIEnv::addVar(int var_id)
     if (!pName)
         return 0;
     char *pValue = achBuf;
-    memccpy(pValue, m_pSession->getReq()->getSSIRuntime()
+    memccpy(pValue, m_pSession->getSsiRuntime()
             ->getConfig()->getTimeFmt()->c_str(), 0, 4096);
     int valLen = RequestVars::getReqVar(m_pSession, var_id, pValue, 4096);
     return add(pName, nameLen, pValue, valLen);
 }
 
 
-int SSIEngine::processPrintEnv(HttpSession *pSession)
+int SsiEngine::processPrintEnv(HttpSession *pSession)
 {
-    SSIEnv env(pSession);
+    SsiEnv env(pSession);
 
     HttpCgiTool::buildEnv(&env, pSession);
     env.addVar(REF_DATE_GMT);
@@ -465,7 +614,8 @@ int SSIEngine::processPrintEnv(HttpSession *pSession)
 }
 
 
-int SSIEngine::processSet(HttpSession *pSession, SSIComponent *pComponent)
+int SsiEngine::processSet(HttpSession *pSession,
+                          const SsiComponent *pComponent)
 {
     SubstItem *pItem = (SubstItem *)pComponent->getFirstAttr();
     const AutoStr2 *pVarName;
@@ -477,22 +627,24 @@ int SSIEngine::processSet(HttpSession *pSession, SSIComponent *pComponent)
     {
 
         attr = pItem->getSubType();
-        if (attr == SSI_SET_VAR)
+        if (attr == SSI_ATTR_SET_VAR)
         {
             pVarName = pItem->getStr();
             pItem = (SubstItem *) pItem->next();
             if (!pItem)
                 break;
-            if (pItem->getSubType() != SSI_SET_VALUE)
+            if (pItem->getSubType() != SSI_ATTR_SET_VALUE)
                 continue;
             p = achBuf1;
             len = 8192;
             len = RequestVars::appendSubst(pItem, pSession, p, len, 0,
-                                           pSession->getReq()->getSSIRuntime()->getRegexResult(),
-                                           pSession->getReq()->getSSIRuntime()
+                                           pSession->getSsiRuntime()->getRegexResult(),
+                                           pSession->getSsiRuntime()
                                            ->getConfig()->getTimeFmt()->c_str());
-            RequestVars::setEnv(pSession, pVarName->c_str(), pVarName->len(), achBuf1,
-                                p - achBuf1);
+            pSession->getSsiRuntime()->setVar(pVarName->c_str(), pVarName->len(),
+                                              achBuf1, p - achBuf1);
+//             RequestVars::setEnv(pSession, pVarName->c_str(), pVarName->len(), achBuf1,
+//                                 p - achBuf1);
         }
 
         pItem = (SubstItem *) pItem->next();
@@ -501,7 +653,7 @@ int SSIEngine::processSet(HttpSession *pSession, SSIComponent *pComponent)
 }
 
 
-int SSIEngine::appendLocation(HttpSession *pSession, const char *pLocation,
+int SsiEngine::appendLocation(HttpSession *pSession, const char *pLocation,
                               int len)
 {
     char achBuf[40960];
@@ -524,8 +676,10 @@ static int  shortCurcuit(ExprToken *&pTok)
 {
     if (!pTok)
         return 0;
-    pTok = pTok->next();
     int type = pTok->getType();
+    pTok = pTok->next();
+    if (!pTok)
+        return 0;
     switch (type)
     {
     case ExprToken::EXP_STRING:
@@ -572,7 +726,7 @@ static int compString(HttpSession *pSession, int type, ExprToken *&pTok)
         p1 = achBuf1;
         len = 40960;
         RequestVars::buildString(pTok->getFormat(), pSession, p1, len, 0,
-                                 pSession->getReq()->getSSIRuntime()->getRegexResult());
+                                 pSession->getSsiRuntime()->getRegexResult());
         p1 = achBuf1;
     }
     pTok = pTok->next();
@@ -580,7 +734,7 @@ static int compString(HttpSession *pSession, int type, ExprToken *&pTok)
         return 0;
     if (pTok->getType() == ExprToken::EXP_REGEX)
     {
-        ret = pSession->getReq()->getSSIRuntime()->execRegex(pTok->getRegex(), p1,
+        ret = pSession->getSsiRuntime()->execRegex(pTok->getRegex(), p1,
                 len);
         if (ret == 0)
             ret = 10;
@@ -599,7 +753,7 @@ static int compString(HttpSession *pSession, int type, ExprToken *&pTok)
         p2 = achBuf2;
         len = 40960;
         RequestVars::buildString(pTok->getFormat(), pSession, p2, len, 0,
-                                 pSession->getReq()->getSSIRuntime()->getRegexResult());
+                                 pSession->getSsiRuntime()->getRegexResult());
         p2 = achBuf2;
     }
     pTok = pTok->next();
@@ -623,7 +777,7 @@ static int compString(HttpSession *pSession, int type, ExprToken *&pTok)
 }
 
 
-int SSIEngine::evalOperator(HttpSession *pSession, ExprToken *&pTok)
+int SsiEngine::evalOperator(HttpSession *pSession, ExprToken *&pTok)
 {
     char   *p1 = NULL;
     int     len;
@@ -643,7 +797,7 @@ int SSIEngine::evalOperator(HttpSession *pSession, ExprToken *&pTok)
             p1 = achBuf1;
             len = 40960;
             RequestVars::buildString(pCur->getFormat(), pSession, p1, len, 0,
-                                     pSession->getReq()->getSSIRuntime()->getRegexResult());
+                                     pSession->getSsiRuntime()->getRegexResult());
             return len > 0;
         }
     case ExprToken::EXP_REGEX:
@@ -661,14 +815,13 @@ int SSIEngine::evalOperator(HttpSession *pSession, ExprToken *&pTok)
         (type == ExprToken::EXP_OR))
     {
         ret = evalOperator(pSession, pTok);
-        if (((ret) && (pTok->getType() == ExprToken::EXP_OR)) ||
-            ((!ret) && (pTok->getType() == ExprToken::EXP_AND)))
-        {
-            if (pTok)
-                shortCurcuit(pTok);
-            return ret;
-        }
-        ret = evalOperator(pSession, pTok);
+        if (!pTok)
+            return 0;
+        if (((ret) && (type == ExprToken::EXP_OR)) ||
+            ((!ret) && (type == ExprToken::EXP_AND)))
+            shortCurcuit(pTok);
+        else
+            ret = evalOperator(pSession, pTok);
         return ret;
     }
     if (!pTok)
@@ -677,7 +830,7 @@ int SSIEngine::evalOperator(HttpSession *pSession, ExprToken *&pTok)
 }
 
 
-int SSIEngine::evalExpr(HttpSession *pSession, SubstItem *pItem)
+int SsiEngine::evalExpr(HttpSession *pSession, SubstItem *pItem)
 {
     int ret = 0;
     if ((!pItem) || (pItem->getType() != REF_EXPR))
@@ -689,29 +842,54 @@ int SSIEngine::evalExpr(HttpSession *pSession, SubstItem *pItem)
 }
 
 
-int SSIEngine::processIf(HttpSession *pSession, SSI_If *pComponent)
+int SsiEngine::pushBlock(HttpSession *pSession,
+                         const SsiComponent *pComponent)
 {
-    SubstItem *pItem = (SubstItem *)pComponent->getFirstAttr();
-    int ret = evalExpr(pSession, pItem);
-    SSIBlock *pBlock;
-    if (ret)
-        pBlock = pComponent->getIfBlock();
-    else
-        pBlock = pComponent->getElseBlock();
-    if (pBlock != NULL)
-    {
-        SSIScript *pScript = pSession->getReq()->getSSIRuntime()
-                             ->getCurrentScript();
-        pScript->setCurrentBlock(pBlock);
-    }
+    const SsiBlock *pBlock = (const SsiBlock *)pComponent;
+    pSession->getSsiStack()->setCurrentBlock(pBlock);
     return 0;
 }
 
 
-int SSIEngine::executeComponent(HttpSession *pSession,
-                                SSIComponent *pComponent)
+int SsiEngine::processConditional(HttpSession *pSession,
+                                  const SsiComponent *pComponent)
+{
+    SubstItem *pItem = (SubstItem *)pComponent->getFirstAttr();
+    int ret = evalExpr(pSession, pItem);
+    if (ret)
+        pushBlock(pSession, pComponent);
+    return 0;
+}
+
+
+int SsiEngine::processSubSessionRet(int ret, HttpSession *pSession,
+                                    HttpSession *pSubSession)
+{
+    if (ret)
+    {
+        //Got error in processing code
+        //generate error response
+        printError(pSession, NULL);
+        pSession->closeSubSession(pSubSession);
+        return 0;
+    }
+    else
+    {
+        pSession->attachSubSession(pSubSession);
+        if (pSubSession->getState() == HSS_COMPLETE)
+        {
+            pSession->closeSubSession(pSubSession);
+            return 0;
+        }
+    }
+    return -2;
+
+}
+
+
+int SsiEngine::executeComponent(HttpSession *pSession,
+                                const SsiComponent *pComponent)
 {
-    AutoBuf *pBuf;
     int ret;
 
     LS_DBG_H(pSession->getLogSession(), "SSI Process component: %d",
@@ -719,81 +897,128 @@ int SSIEngine::executeComponent(HttpSession *pSession,
 
     switch (pComponent->getType())
     {
-    case SSIComponent::SSI_String:
-        pBuf = pComponent->getContentBuf();
-        pSession->appendDynBody(pBuf->begin(), pBuf->size());
+    case SsiComponent::SSI_String:
+        pSession->appendDynBody(pSession->getSsiStack()->getScript()->getContent(),
+                pComponent->getContentOffset(), pComponent->getContentLen());
         break;
-    case SSIComponent::SSI_Config:
-        updateSSIConfig(pSession, pComponent, pSession->getReq()->getSSIRuntime());
+    case SsiComponent::SSI_Config:
+        updateSsiConfig(pSession, pComponent, pSession->getSsiRuntime());
         break;
-    case SSIComponent::SSI_Echo:
+    case SsiComponent::SSI_Echo:
         processEcho(pSession, pComponent);
         break;
-    case SSIComponent::SSI_Exec:
+    case SsiComponent::SSI_Exec:
         ret = processExec(pSession, pComponent);
         return ret;
         break;
-    case SSIComponent::SSI_FSize:
-    case SSIComponent::SSI_Flastmod:
+    case SsiComponent::SSI_FSize:
+    case SsiComponent::SSI_Flastmod:
         processFileAttr(pSession, pComponent);
         break;
-    case SSIComponent::SSI_Include:
+    case SsiComponent::SSI_Include:
         ret = processInclude(pSession, pComponent);
         return ret;
         break;
-    case SSIComponent::SSI_Printenv:
+    case SsiComponent::SSI_Printenv:
         processPrintEnv(pSession);
         break;
-    case SSIComponent::SSI_Set:
+    case SsiComponent::SSI_Set:
         processSet(pSession, pComponent);
         break;
-    case SSIComponent::SSI_If:
-    case SSIComponent::SSI_Elif:
-        processIf(pSession, (SSI_If *)pComponent);
+    case SsiComponent::SSI_Switch:
+        pushBlock(pSession, pComponent);
+        break;
+    case SsiComponent::SSI_If:
+    case SsiComponent::SSI_Elif:
+        processConditional(pSession, pComponent);
+        break;
+    case SsiComponent::SSI_Else:
+        pushBlock(pSession, pComponent);
         break;
-        //SSI_Else,
-        //SSI_Elif,
-        //SSI_Endif,
 
     }
     return 0;
 }
 
 
-int SSIEngine::endExecute(HttpSession *pSession)
+int SsiEngine::endExecute(HttpSession *pSession)
 {
-    SSIRuntime *pRuntime = pSession->getReq()->getSSIRuntime();
-    if (pRuntime)
-        delete pRuntime;
-    pSession->getReq()->setSSIRuntime(NULL);
+    pSession->releaseSsiRuntime();
     return 0;
 }
 
 
-int SSIEngine::resumeExecute(HttpSession *pSession)
+int SsiEngine::resumeExecute(HttpSession *pSession)
 {
-    SSIRuntime *pRuntime = pSession->getReq()->getSSIRuntime();
+    SsiStack *pStack = pSession->getSsiStack();
+    if (pSession->getSsiRuntime() && pSession->getSsiRuntime()->isInSsiEngine())
+        return 0;
+    pStack->nextComponentOfCurScript();
+    return execute(pSession);
+    //TODO: redo this when I get to it.  Left pRuntime so I have a warning here.
+//     SsiRuntime *pRuntime = pSession->getSsiRuntime();
+//     int ret = 0;
+//     if (!pRuntime)
+//         return LS_FAIL;
+//     SsiComponent *pComponent;
+//     while (!pRuntime->done())
+//     {
+//         SsiScript *pScript = pRuntime->getCurrentScript();
+//         pRuntime->clearFlag();
+//         pComponent = pScript->nextComponent();
+//         if (!pComponent)
+//         {
+//             pRuntime->pop();
+//             if (!pRuntime->done())
+//                 pSession->getReq()->restorePathInfo();
+//             continue;
+//         }
+//         ret = executeComponent(pSession, pComponent);
+//         if (ret == -2)
+//             return 0;
+//     }
+//     endExecute(pSession);
+//     return pSession->endResponse(1);
+
+}
+
+
+int SsiEngine::execute(HttpSession *pSession)
+{
+    SsiStack *pStack = pSession->getSsiStack();
     int ret = 0;
-    if (!pRuntime)
-        return LS_FAIL;
-    SSIComponent *pComponent;
-    while (!pRuntime->done())
+    if (!pStack)
+        return -1;
+    if (!pSession->isNoRespBody())
     {
-        SSIScript *pScript = pRuntime->getCurrentScript();
-        pRuntime->clearFlag();
-        pComponent = pScript->nextComponent();
-        if (!pComponent)
+        pSession->getSsiRuntime()->incInSsiEngine();
+        const SsiComponent *pComponent = pStack->getCurrentComponent();
+        while (pComponent || pStack->getPrevious())
         {
-            pRuntime->pop();
-            if (!pRuntime->done())
-                pSession->getReq()->restorePathInfo();
-            continue;
+            if (pComponent)
+            {
+                pStack->clearFlag();
+                ret = executeComponent(pSession, pComponent);
+                if (ret == -2 || pSession->getSsiRuntime() == NULL)
+                {
+                    if (pSession->getSsiRuntime())
+                        pSession->getSsiRuntime()->decInSsiEngine();
+                    return 0;
+                }
+                pStack = pSession->getSsiStack();
+                pComponent = pStack->nextComponentOfCurScript();
+            }
+            while (!pComponent && pStack->getPrevious())
+            {
+                pStack = pSession->popSsiStack();
+                if (pStack)
+                    pComponent = pStack->nextComponentOfCurScript();
+            }
         }
-        ret = executeComponent(pSession, pComponent);
-        if (ret == -2)
-            return 0;
+        if (pSession->getSsiRuntime())
+            pSession->getSsiRuntime()->decInSsiEngine();
     }
     endExecute(pSession);
-    return pSession->endResponse(1);
-
+    pSession->endResponse(1);
+    return 0;
 }
