@@ -9,12 +9,16 @@
 #include <sys/types.h>
 #include <unistd.h>
 #include <sys/socket.h>
+#include <sys/uio.h>
 #include <poll.h>
 #include <time.h>
 #include <fcntl.h>
+
+#include <lsdef.h>
 #include <lsr/ls_pool.h>
 #include <socket/ls_sock.h>
 #include <sslpp/ls_fdbuf_bio.h>
+#include <openssl/bio.h>
 
 #ifdef RUN_TEST
 #define DEBUGGING
@@ -38,7 +42,7 @@
 
 #endif
 
-const int RBIO_BUF_SIZE = 2048; // Initial size
+const int RBIO_BUF_SIZE = 4096; // Initial size
 #if OPENSSL_VERSION_NUMBER < 0x10100000
 BIO_METHOD  s_biom_s;
 #endif
@@ -57,7 +61,11 @@ static int BIO_fd_should_retry(int i);
 static int BIO_fd_non_fatal_error(int err);
 #endif
 
-#define SIMPLE_RETRY(err) ((err == EAGAIN) || (err = EWOULDBLOCK))
+#if EWOULDBLOCK != EAGAIN
+#define SIMPLE_RETRY(err) ((err == EAGAIN) || (err == EWOULDBLOCK))
+#else
+#define SIMPLE_RETRY(err) (err == EAGAIN)
+#endif
 
 #if OPENSSL_VERSION_NUMBER >= 0x10100000
 void *BIO_get_data(BIO *a);
@@ -77,7 +85,7 @@ void ls_fdbuf_bio_init(ls_fdbio_data *fdbio)
 static int bio_fd_free(BIO *a)
 {
     ls_fdbio_data *fdbio;
-    DEBUG_MESSAGE("[BIO: %d] bio_fd_free: %p\n", getpid(), a);
+    DEBUG_MESSAGE("[BIO] bio_fd_free: %p\n", a);
     if (a == NULL)
         return 0;
     fdbio = LS_FDBUF_FROM_BIO(a);
@@ -99,17 +107,17 @@ static int bio_fd_read(BIO *b, char *out, int outl)
     ls_fdbio_data *fdbio = LS_FDBUF_FROM_BIO(b);
     int fd = BIO_get_fd(b, 0);
     int total = 0;
-    DEBUG_MESSAGE("[BIO: %d] bio_fd_read: %p, %d bytes on %d\n", getpid(), b, 
+    DEBUG_MESSAGE("[BIO] bio_fd_read: %p, %d bytes on %d\n", b,
                   outl, fd);
     if ((out == NULL) || (!outl))
     {
-        INFO_MESSAGE("[BIO: %d] bio_fd_read NO BUFFER!!\n", getpid());
+        INFO_MESSAGE("[BIO] bio_fd_read NO BUFFER!!\n");
         err = ENOMEM;
         return -1;
     }
     if (fdbio->m_rbioClosed)
     {
-        DEBUG_MESSAGE("[BIO: %d] bio_fd_read: CLOSED ON PREVIOUS READ\n", getpid());
+        DEBUG_MESSAGE("[BIO] bio_fd_read: CLOSED ON PREVIOUS READ\n");
         errno = 0;
         return 0;
     }
@@ -121,15 +129,15 @@ static int bio_fd_read(BIO *b, char *out, int outl)
         if (buffered)
         {
             copy = (buffered > rd_remaining) ? rd_remaining : buffered;
-            DEBUG_MESSAGE("[BIO: %d] bio_fd_read: Use buffered %d of %d\n", 
-                          getpid(), copy, rd_remaining);
+            DEBUG_MESSAGE("[BIO] bio_fd_read: Use buffered %d of %d\n",
+                          copy, rd_remaining);
             memcpy(&out[total], &fdbio->m_rbioBuf[fdbio->m_rbioIndex], copy);
             fdbio->m_rbioIndex += copy;
             total += copy;
             if (total >= outl)
             {
-                DEBUG_MESSAGE("[BIO: %d] bio_fd_read: GOT TOTAL %d\n", 
-                              getpid(), total);
+                DEBUG_MESSAGE("[BIO] bio_fd_read: GOT TOTAL %d\n",
+                              total);
                 return total;
             }
             fdbio->m_rbioBuffered = 0;
@@ -138,8 +146,7 @@ static int bio_fd_read(BIO *b, char *out, int outl)
         }
         if ((rd_remaining < fdbio->m_rbioBufSz) && (fdbio->m_rbioBuf))
         {
-            DEBUG_MESSAGE("[BIO: %d] bio_fd_read: Read into buffer\n", 
-                          getpid());
+            DEBUG_MESSAGE("[BIO] bio_fd_read: Read into buffer\n");
             ret = ls_read(fd, fdbio->m_rbioBuf, fdbio->m_rbioBufSz);
             if (ret < fdbio->m_rbioBufSz)
                 fdbio->m_rbioWaitEvent = 1;
@@ -147,8 +154,7 @@ static int bio_fd_read(BIO *b, char *out, int outl)
         }
         else
         {
-            DEBUG_MESSAGE("[BIO: %d] bio_fd_read: Read into data\n", 
-                          getpid());
+            DEBUG_MESSAGE("[BIO] bio_fd_read: Read into data\n");
             ret = ls_read(fd, out + total, rd_remaining);
             if (ret < rd_remaining)
                 fdbio->m_rbioWaitEvent = 1;
@@ -156,24 +162,23 @@ static int bio_fd_read(BIO *b, char *out, int outl)
         }
         if (ret == 0)
         {
-            DEBUG_MESSAGE("[BIO: %d] bio_fd_read: CLOSED\n", getpid());
+            DEBUG_MESSAGE("[BIO] bio_fd_read: CLOSED\n");
             fdbio->m_rbioClosed = 1;
             errno = 0;
             return total;
         }
         if (ret < 0) 
         {
-            errno = err;
+            err = errno;
             if (total)
             {
-                DEBUG_MESSAGE("[BIO: %d] bio_fd_read: error but I have data\n", 
-                              getpid());
+                DEBUG_MESSAGE("[BIO] bio_fd_read: error but I have data\n");
                 return total;
             }
             if ((SIMPLE_RETRY(err)) || (BIO_fd_should_retry(ret)))
             {
-                DEBUG_MESSAGE("[BIO: %d] bio_fd_read: set retry read"
-                              " errno: %d\n", getpid(), err);
+                DEBUG_MESSAGE("[BIO] bio_fd_read: set retry read"
+                              " errno: %d\n", err);
                 errno = err;
                 BIO_set_retry_read(b);
             }
@@ -183,29 +188,202 @@ static int bio_fd_read(BIO *b, char *out, int outl)
         {
             fdbio->m_rbioBuffered = buffered;
             fdbio->m_rbioIndex = 0;
-            DEBUG_MESSAGE("[BIO: %d] bio_fd_read: Preserve read: %d\n", 
-                          getpid(), ret);
+            DEBUG_MESSAGE("[BIO] bio_fd_read: Preserve read: %d\n", ret);
         }
         else
         {
             total += ret;
-            DEBUG_MESSAGE("[BIO: %d] bio_fd_read: Read into data: %d\n", 
-                          getpid(), ret);
+            DEBUG_MESSAGE("[BIO] bio_fd_read: Read into data: %d\n", ret);
         }
     }
     return total;
 }
 
 
+#define LS_FDBIO_FULL           1
+#define LS_FDBIO_BUFFERING      2
+int ls_fdbio_alloc_wbuff(ls_fdbio_data *fdbio, int size)
+{
+    DEBUG_MESSAGE("[FDBIO] alloc write buf %d\n", size);
+    assert(fdbio->m_wbuf_used == 0);
+    if (fdbio->m_wbuf && fdbio->m_wbuf_size >= size)
+        return LS_OK;
+    void *buf = ls_palloc_slab(size);
+    if (buf)
+    {
+        if (fdbio->m_wbuf)
+            ls_pfree_slab(fdbio->m_wbuf, fdbio->m_wbuf_size);
+        fdbio->m_wbuf = (uint8_t *)buf;
+        fdbio->m_wbuf_size = size;
+        fdbio->m_wbuf_sent = 0;
+        return LS_OK;
+    }
+    return LS_FAIL;
+}
+
+
+int ls_fdbio_is_idle(ls_fdbio_data *fdbio)
+{
+    return fdbio->m_wbuf && fdbio->m_wbuf_used == 0;
+}
+
+
+void ls_fdbio_relelase_wbuff(ls_fdbio_data *fdbio)
+{
+    assert(fdbio->m_wbuf_used == 0);
+    DEBUG_MESSAGE("[FDBIO] free buf %d\n", (int)fdbio->m_wbuf_size);
+    if (fdbio->m_wbuf == NULL)
+        return;
+    ls_pfree_slab(fdbio->m_wbuf, fdbio->m_wbuf_size);
+    fdbio->m_wbuf = NULL;
+}
+
+
+static int ls_fdbio_combine_write(ls_fdbio_data *fdbio, int fd, const void *buf, int num)
+{
+    DEBUG_MESSAGE("[FDBIO] ls_fdbio_combine_write, to write: %d, used: %d, sent: %d\n",
+           num, (int)fdbio->m_wbuf_used, (int)fdbio->m_wbuf_sent);
+    struct iovec iov[2];
+    iov[0].iov_base = fdbio->m_wbuf + fdbio->m_wbuf_sent;
+    iov[0].iov_len = fdbio->m_wbuf_used - fdbio->m_wbuf_sent;
+    iov[1].iov_base = (void *)buf;
+    iov[1].iov_len = num;
+    int ret = ls_writev(fd, iov, 2);
+    DEBUG_MESSAGE("[FDBIO] ls_writev(%d + %d) ret: %d\n",
+                  iov[0].iov_len, iov[1].iov_len, ret);
+    if (ret > 0)
+    {
+        if (ret >= (int)iov[0].iov_len)
+        {
+            ret -= iov[0].iov_len;
+            fdbio->m_wbuf_used = 0;
+            fdbio->m_wbuf_sent = 0;
+            if (ret == 0)
+            {
+                ret = -1;
+                errno = EWOULDBLOCK;
+            }
+        }
+        else
+        {
+            fdbio->m_wbuf_sent += ret;
+            ret = -1;
+            errno = EWOULDBLOCK;
+        }
+    }
+    DEBUG_MESSAGE("[FDBIO] combine_write ret: %d, buffer used: %d, sent: %d\n",
+                  ret, fdbio->m_wbuf_used, fdbio->m_wbuf_sent);
+    return ret;
+}
+
+
+static int ls_fdbio_buff_write(ls_fdbio_data *fdbio, int fd, const void *buf, int num)
+{
+    DEBUG_MESSAGE("[FDBIO] ls_fdbio_buff_write, to write: %d, used: %d, sent: %d\n",
+           num, (int)fdbio->m_wbuf_used, (int)fdbio->m_wbuf_sent);
+    if (fdbio->m_flag & LS_FDBIO_FULL)
+        return 0;
+    if (!fdbio->m_wbuf)
+    {
+        if (num < 4096)
+            if (ls_fdbio_alloc_wbuff(fdbio, 4096) == LS_FAIL)
+                return LS_FAIL;
+    }
+
+    int avail = fdbio->m_wbuf_size - fdbio->m_wbuf_used;
+    if (avail < num)
+    {
+        int ret;
+        if (fdbio->m_wbuf_used > 0)
+            ret = ls_fdbio_combine_write(fdbio, fd, buf, num);
+        else
+            ret = ls_write(fd, buf, num);
+        if (ret >= num && fdbio->m_wbuf_size < 16413 * 4)
+        {
+            int new_size = fdbio->m_wbuf_size + 16413;
+            if (new_size < 16413 * 2)
+                new_size = 16413 * 2;
+            ls_fdbio_alloc_wbuff(fdbio, new_size);
+        }
+        return ret;
+    }
+    memmove(fdbio->m_wbuf + fdbio->m_wbuf_used, buf, num);
+    fdbio->m_wbuf_used += num;
+    DEBUG_MESSAGE("[FDBIO] lstls_buff_write, to write: %d, finished: %ld, used: %d, sent: %d\n",
+           num, num, (int)fdbio->m_wbuf_used, (int)fdbio->m_wbuf_sent);
+    return num;
+}
+
+
+int ls_fdbio_is_full(ls_fdbio_data *fdbio)
+{   return fdbio->m_flag & LS_FDBIO_FULL;      }
+
+
+void ls_fdbio_clear_buff_full(ls_fdbio_data *fdbio)
+{    fdbio->m_flag &= ~LS_FDBIO_FULL;    }
+
+
+void ls_fdbio_set_wbuff(ls_fdbio_data *fdbio, int dobuff)
+{
+    if (dobuff)
+        fdbio->m_flag |= LS_FDBIO_BUFFERING;
+    else
+        fdbio->m_flag &= ~LS_FDBIO_BUFFERING;
+}
+
+
+int ls_fdbio_flush(ls_fdbio_data *fdbio, int fd)
+{
+    if (fdbio->m_flag & LS_FDBIO_FULL)
+        return 0;
+    int pending = fdbio->m_wbuf_used - fdbio->m_wbuf_sent;
+    if (pending <= 0)
+        return 1;
+    int ret = ls_write(fd, fdbio->m_wbuf + fdbio->m_wbuf_sent, pending);
+    DEBUG_MESSAGE("[FDBIO] flush_ex write(%d, %p, %d) return %d\n",
+           fd, fdbio->m_wbuf + fdbio->m_wbuf_sent, pending, ret);
+    if (ret >= pending)
+    {
+        fdbio->m_wbuf_used = 0;
+        fdbio->m_wbuf_sent = 0;
+        DEBUG_MESSAGE("[FDBIO] FLUSHED\n");
+        return 1;
+    }
+    else
+    {
+        DEBUG_MESSAGE("[FDBIO] partial write, mark buffer FULL.\n");
+        fdbio->m_flag |= LS_FDBIO_FULL;
+        if (ret > 0)
+        {
+            fdbio->m_wbuf_sent += ret;
+        }
+        else if (ret == -1 && errno != EAGAIN && errno != EWOULDBLOCK)
+        {
+            return -1;
+        }
+    }
+    return 0;
+}
+
+
 static int bio_fd_write(BIO *b, const char *in, int inl)
 {
     int ret;
     int err = 0;
     int fd = BIO_get_fd(b, 0);
+    ls_fdbio_data *fdbio = LS_FDBUF_FROM_BIO(b);
 
-    DEBUG_MESSAGE("[BIO: %d] bio_fd_write: %p, %d bytes on %d\n", getpid(), b, 
-                  inl, fd);
-    ret = ls_write(fd, (void *)in, inl);
+    DEBUG_MESSAGE("[FDBIO] bio_fd_write: %p, %d bytes on %d\n", b, inl, fd);
+    if (fdbio->m_flag & LS_FDBIO_BUFFERING)
+    {
+        ret = ls_fdbio_buff_write(fdbio, fd, in, inl);
+    }
+    else if (fdbio->m_wbuf_used > 0)
+    {
+        ret = ls_fdbio_combine_write(fdbio, fd, in, inl);
+    }
+    else
+        ret = ls_write(fd, (void *)in, inl);
     if ( ret > 0)
     {
         BIO_clear_retry_flags(b);
@@ -214,16 +392,16 @@ static int bio_fd_write(BIO *b, const char *in, int inl)
     err = errno;
     if ((ret == -1) && (!(SIMPLE_RETRY(err))) && (!BIO_fd_should_retry(ret)))
     {
-        DEBUG_MESSAGE("[BIO: %d] bio_fd_write: actual write failed, errno: %d\n",
-                      getpid(), err);
+        DEBUG_MESSAGE("[FDBIO] bio_fd_write: actual write failed, errno: %d\n",
+                      err);
         return ret;
     }            
     BIO_clear_retry_flags(b);
     if (ret < 0) {
         if ((BIO_fd_should_retry(ret)) || (SIMPLE_RETRY(err)))
         {
-            DEBUG_MESSAGE("[BIO: %d] bio_fd_write: %p, early return, errno: %d,"
-                          " ret: %d\n", getpid(), b, err, ret);
+            DEBUG_MESSAGE("[FDBIO] bio_fd_write: %p, early return, errno: %d,"
+                          " ret: %d\n", b, err, ret);
             BIO_set_retry_write(b);
             errno = EAGAIN;
             //return bio_fd_write(b, in, inl);
@@ -231,8 +409,7 @@ static int bio_fd_write(BIO *b, const char *in, int inl)
         }
     }
     
-    DEBUG_MESSAGE("[BIO: %d] bio_fd_write: %p, returning: %d\n", getpid(), b, 
-                  ret);
+    DEBUG_MESSAGE("[FDBIO] bio_fd_write: %p, returning: %d\n", b, ret);
     errno = err;
     return ret;
 }
@@ -242,7 +419,7 @@ static int bio_fd_puts(BIO *bp, const char *str)
 {
     int n, ret;
     
-    DEBUG_MESSAGE("[BIO: %d] bio_fd_puts: %p, %s\n", getpid(), bp, str);
+    DEBUG_MESSAGE("[FDBIO] bio_fd_puts: %p, %s\n", bp, str);
 
     n = strlen(str);
     ret = bio_fd_write(bp, str, n);
@@ -256,7 +433,7 @@ static int bio_fd_gets(BIO *bp, char *buf, int size)
     char *ptr = buf;
     char *end = buf + size - 1;
     
-    DEBUG_MESSAGE("[BIO: %d] bio_fd_gets: %p\n", getpid(), bp);
+    DEBUG_MESSAGE("[BIO] bio_fd_gets: %p\n", bp);
 
     while (ptr < end && bio_fd_read(bp, ptr, 1) > 0) {
         if (*ptr++ == '\n')
@@ -335,7 +512,7 @@ static int setup_writes(ls_fdbio_data *fdbio)
 {
     if (!fdbio)
         return 0;
-    DEBUG_MESSAGE("[BIO: %d] setup_writes\n", getpid());
+    DEBUG_MESSAGE("[BIO] setup_writes\n");
     return 0;
 }
 
@@ -344,7 +521,7 @@ BIO *ls_fdbio_create(int fd, ls_fdbio_data *fdbio)
 {
     if (!s_biom)
     {
-        DEBUG_MESSAGE("[BIO: %d] ls_fdbio_create METHOD\n", getpid());
+        DEBUG_MESSAGE("[BIO] ls_fdbio_create METHOD\n");
     
         s_biom_fd_builtin = BIO_s_fd();
 #if OPENSSL_VERSION_NUMBER >= 0x10100000
@@ -352,8 +529,7 @@ BIO *ls_fdbio_create(int fd, ls_fdbio_data *fdbio)
             BIO_TYPE_FD | BIO_TYPE_SOURCE_SINK | BIO_TYPE_DESCRIPTOR, 
             "Litespeed BIO Method")))
         {
-            DEBUG_MESSAGE("[BIO: %d] ERROR creating BIO_METHOD\n", 
-                          getpid());
+            DEBUG_MESSAGE("[BIO] ERROR creating BIO_METHOD\n");
             return NULL;
         }
 #ifdef OPENSSL_IS_BORINGSSL
@@ -380,7 +556,8 @@ BIO *ls_fdbio_create(int fd, ls_fdbio_data *fdbio)
     if (!fdbio->m_rbioBufSz)
         fdbio->m_rbioBufSz = RBIO_BUF_SIZE;
     
-    if (!(fdbio->m_rbioBuf = (char *)ls_palloc(fdbio->m_rbioBufSz)))
+    if (!fdbio->m_rbioBuf
+        && !(fdbio->m_rbioBuf = (char *)ls_palloc(fdbio->m_rbioBufSz)))
     {
         DEBUG_MESSAGE("Insufficient memory allocating %d bytes\n",
                       fdbio->m_rbioBufSz);
@@ -393,8 +570,8 @@ BIO *ls_fdbio_create(int fd, ls_fdbio_data *fdbio)
     BIO *bio = BIO_new(s_biom);
     if (!bio)
     {
-        DEBUG_MESSAGE("[BIO: %d:%p] ls_fdbio_create error creating rbio\n",
-                      getpid(), fdbio);
+        DEBUG_MESSAGE("[BIO: %p] ls_fdbio_create error creating rbio\n",
+                       fdbio);
         // everything freed in the destructor
         return NULL;
     }
@@ -404,8 +581,8 @@ BIO *ls_fdbio_create(int fd, ls_fdbio_data *fdbio)
 #else
     BIO_set_app_data(bio, fdbio);
 #endif    
-    DEBUG_MESSAGE("[BIO: %d] ls_fdbio_create bio: %p\n",
-                  getpid(), bio);
+    DEBUG_MESSAGE("[BIO] ls_fdbio_create bio: %p\n",
+                   bio);
     
     setup_writes(fdbio);
     return bio;
