@@ -6,14 +6,12 @@
 #ifndef __LS_FDBUF_BIO_H__
 #define __LS_FDBUF_BIO_H__
 
+#include <stdint.h>
+
 #ifdef __cplusplus 
 extern "C" {
 #endif
 
-#include <openssl/ssl.h>
-#include <openssl/bio.h>
-#include <openssl/err.h>
-
 /**
  * @file ls_fdbuf_bio.h
  * @brief Include file for BIO library to separate BIO from SSL so that porting
@@ -27,6 +25,12 @@ typedef struct ls_fdbio_data {
     uint16_t     m_rbioBufSz;
     uint8_t      m_rbioClosed;
     uint8_t      m_rbioWaitEvent;
+
+    uint8_t     *m_wbuf;
+    int32_t      m_wbuf_size;
+    int32_t      m_wbuf_used;
+    int32_t      m_wbuf_sent;
+    int32_t      m_flag;
 } ls_fdbio_data;
 
 
@@ -47,8 +51,11 @@ void ls_fdbuf_bio_init(ls_fdbio_data *fdbio);
  * Allocated by caller.
  * @return NULL to indicate an error.
  */
-BIO *ls_fdbio_create(int fd, ls_fdbio_data *fdbio);
+struct bio_st *ls_fdbio_create(int fd, ls_fdbio_data *fdbio);
 
+int ls_fdbio_flush(ls_fdbio_data *fdbio, int fd);
+
+void ls_fdbio_set_wbuff(ls_fdbio_data *fdbio, int dobuff);
 
 #ifdef __cplusplus 
 }
