@@ -29,6 +29,11 @@
 #include <util/ssnprintf.h>
 #include "stringtool.h"
 
+#include <assert.h>
+#if __cplusplus <= 199711L && !defined(static_assert)
+#define static_assert(a, b) _Static_assert(a, b)
+#endif
+
 #include <openssl/ssl.h>
 #include <openssl/x509_vfy.h>
 
@@ -531,11 +536,16 @@ int HttpFetch::buildReq(const char *pMethod, const char *pURL,
                                   );
     if (m_pExtraReqHdrs != NULL)
     {
+        int trim = 0;
+        if (extraLen >= 2
+                && *(m_pExtraReqHdrs + extraLen - 2) == '\r'
+                && *(m_pExtraReqHdrs + extraLen - 1) == '\n')
+            trim = 2;
+
         m_reqHeaderLen += safe_snprintf(m_pReqBuf + m_reqHeaderLen,
                                     len - m_reqHeaderLen,
                                     "%.*s\r\n",
-                                    extraLen, m_pExtraReqHdrs
-                                    );
+                                    extraLen - trim, m_pExtraReqHdrs);
     }
     if (m_reqBodyLen > 0)
     {
